import brs.*;
import genRac.*;

public class TreeTraversal {

    /**
     * Traverse a BiTree using a RAC to control the order of traversal.
     *
     * @param root the BiTree to traverse
     * @param factory a factory for creating a RAC
     */
    public static void traverse(BiTree root, IRACFactory<BiTree> factory) {
        final IRAContainer<BiTree> rac = factory.makeRAC();

        /**
         * If root is non-empty, add it to rac.
         */
        root.execute(new IVisitor (){
            public Object emptyCase(BiTree host, Object... input) {
                return null;
            }

            public Object nonEmptyCase(BiTree host, Object... input) {
                rac.put(host);
                return null;
            }
        });

        System.out.print("Traversal:");

        while (!rac.isEmpty()) {
            BiTree bt = rac.get();

            System.out.print(" " + bt.getRootDat());

            /**
             * If bt's left child is non-empty, add it to rac.
             */
            bt.getLeftSubTree().execute(new IVisitor (){
                public Object emptyCase(BiTree host, Object... input) {
                    return null;
                }

                public Object nonEmptyCase(BiTree host, Object... input) {
                    rac.put(host);
                    return null;
                }
            });

            /**
             * If bt's right child is non-empty, add it to rac.
             */
            bt.getRightSubTree().execute(new IVisitor (){
                public Object emptyCase(BiTree host, Object... input) {
                    return null;
                }

                public Object nonEmptyCase(BiTree host, Object... input) {
                    rac.put(host);
                    return null;
                }
            });
        }

        System.out.println("");
    }
}
