package lrs.visitor;
import lrs.*;

/**
 * Weaves ("Zips") together any two LRStructs even if they
 * share any node to yield two completely distinct LRStructs 
 * with the same data.
 * For example, when we zip (5, 3, 9), the host, together
 * with (-7, 0, 8, -15), the input parameter, both lists
 * become (5, -7, 3, 0, 9, 8, -15).
 */

public class Zipper2 implements IAlgo {
  public static final Zipper2 Singleton = new Zipper2();
  private Zipper2() {
  }
  
  /**
   * Utility algo to copy the contents of a host list to a given
   * target list
   */
  private static final IAlgo copyToAlgo = new IAlgo() {
    public Object emptyCase(LRStruct host, Object... targetList) {
      return host;
    }

    public Object nonEmptyCase(LRStruct host, Object... targetList) {
      host.getRest().execute(this, targetList);
      ((LRStruct) targetList[0]).insertFront(host.getFirst());
      return host;
    }
  };
  
  /**
   * host is empty, so there is nothing to zip with.
   * Just copy the input list to here
   * @param host empty
   * @param input [0] the other LRStruct.
   */
  public Object emptyCase(LRStruct firstHost, Object... otherList) {
    return ((LRStruct) otherList[0]).execute(copyToAlgo, firstHost);
  }
  
  
  /**
   * 
   * @param host not empty
   * @param input [0] the other LRStruct.
   */
  public Object nonEmptyCase(final LRStruct firstHost, Object... otherList) {
    // Delegate to the other list
    return ((LRStruct)otherList[0]).execute(new IAlgo() {
      
      /**
       * First list is NE, other list is MT.   Copy from first list
       * to this list (other list)
       */
      public Object emptyCase(LRStruct otherHost, Object... nu) {
        firstHost.execute(copyToAlgo, otherHost);
        return firstHost;
      }
      
      /**
       * Both the first list and the other list are NE.
       */
      public Object nonEmptyCase(LRStruct otherHost, Object... nu) {
        // Recur to the rest of the first list with rest of the other list
        firstHost.getRest().execute(Zipper2.this, otherHost.getRest());
        // Insert the first element of the other list into 
        // the front of the rest first list.
        firstHost.getRest().insertFront(otherHost.getFirst());
        // Insert the first element of the first list into 
        // the front of the other list.
        otherHost.insertFront(firstHost.getFirst());
        return firstHost;
      }
      
    });

  }
}

