package lrs.visitor;
import lrs.*;

/**
 * Weaves ("Zips") together any two LRStructs to yield two completely distinct LRStructs 
 * with the same data.
 * For example, when we zip (5, 3, 9), the host, together
 * with (-7, 0, 8, -15), the input parameter, both lists
 * become (5, -7, 3, 0, 9, 8, -15).
 */

public class Zipper3 implements IAlgo {
    public static final Zipper3 Singleton = new Zipper3();
    private Zipper3() {
    }
    
    /**
     * Utility algo to copy the contents of a host list to a given
     * target list
     */
    private static final IAlgo copyToAlgo = new IAlgo() {
        public Object emptyCase(LRStruct host, Object... targetList) {
            return host;
        }
        
        public Object nonEmptyCase(LRStruct host, Object... targetList) {
            host.getRest().execute(this, targetList);
            ((LRStruct) targetList[0]).insertFront(host.getFirst());
            return host;
        }
    };
    
    /**
     * host is empty, so there is nothing to zip with.
     * Just copy the input list to here
     * @param host empty
     * @param input [0] the other LRStruct.
     */
    public Object emptyCase(LRStruct firstHost, Object... otherList) {
        ((LRStruct) otherList[0]).execute(copyToAlgo, firstHost);
        return firstHost;
    }
    
    
    /**
     * 
     * @param host not empty
     * @param input [0] the other LRStruct.
     */
    public Object nonEmptyCase(LRStruct firstHost, Object... otherList) {
        ((LRStruct)otherList[0]).insertFront(firstHost.getFirst());
        
        //Delegate to the other list
        ((LRStruct)otherList[0]).getRest().execute(this, firstHost.getRest());
        return firstHost;
    }
}

